/**
 * @file proprtc.c
 *
 * This program demonstrates the Propeller-GCC RTC library.
 * Default library depends on an RTC COG specified in sys/rtc.h
 *
 */
#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <propeller.h>
#include <sys/rtc.h>

/*
 * forward declarations
 */
void pause(int secs);
void printtime(void);
void settime(int year, int month, int moday, int dst, int hr, int min, int sec);

/**
 * Main program function.
 */
int main(void)
{
    putchar(0); // silence that noisy "QuickStart" board.
    pause(1);   // wait for terminal to start.

    printf("Propeller Library RTC Demo.\n");

    /* test settings */
    //settime(2012, 3, 5, 0, 8, 35, 15);
    //settime(2011, 12, 31, 0, 23, 59, 50);
    //settime(1970, 1, 1, 0, 23, 59, 50);
    settime(2012, 12, 31, 0, 23, 59, 50);

    printf("Verify time set.\n");
    printtime();
    printf("Start COG and wait a second.\n");
    _rtc_start_timekeeping_cog();
    pause(1);
    printtime();
    printf("Now wait 5 seconds.\n");
    pause(5);
    printf("Now print time every second.\n");
    for(;;) {
        printtime();
        pause(1);
    }
    return 0;
}

/**
 * Pause number of seconds based on Propeller system clock.
 * This function is not pthread safe.
 */
void pause(int secs)
{
    while (secs > 0) {
        waitcnt(CLKFREQ+CNT);
        --secs;
    }
}

/**
 * Print library's interpretation of time for human consumption.
 */
void printtime(void)
{
    time_t now;
    now = time(NULL);
    printf("%s", asctime(localtime(&now)));
}

/**
 * Set the user's time which includes year, month, day of month,
 * daylight savings time, hour, minute, seconds.
 *
 * @param year - calendar year to set.
 * @param month - month to set where January is month 1.
 * @param moday - day of the month to set.
 * @param dst - non-zero if daylight savings time in effect.
 * @param hr - hour number to set in 24 hour time 0 to 23.
 * @param min - minute number to set 0 to 59.
 * @param sec - seconds number to set 0 to 59.
 */
void settime(int year, int month, int moday, int dst, int hr, int min, int sec)
{
    struct timeval tv;
    struct tm t;

    t.tm_sec = sec;            /* seconds (0 - 60) */
    t.tm_min = min;            /* minutes (0 - 59) */
    t.tm_hour = hr;            /* hours (0 - 23) */
    t.tm_mday = moday;         /* day of month (1 - 31) */
    t.tm_mon = month-1;        /* month of year (0 - 11) */
    t.tm_year = year-1900;     /* year - 1900 */
    t.tm_isdst = dst ? 1 : 0; /* is daylight savings time */
    tv.tv_sec = mktime(&t);
    settimeofday(&tv, 0);
}
